package com.ann.demo.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import java.io.*;
import java.time.LocalDate;
import java.util.Objects;
import java.util.UUID;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/7/24 16:42
 */
@Component
public class FileUtils {

    static final Logger logger = LoggerFactory.getLogger(FileUtils.class);

    // 错误文件地址
    public static String errorPath;
    @Value("${file.errorPath}")
    public void setErrorPath(String errorPath) {
        FileUtils.errorPath = errorPath;
    }
    public static String getErrorPath() {
        return errorPath;
    }

    // pdf地址
    public static String pdfPath;
    @Value("${file.pdfPath}")
    public void setPdfPath(String pdfPath) {
        FileUtils.pdfPath = pdfPath;
    }
    public static String getPdfPath() {
        return pdfPath;
    }

    //将文件转换成Byte数组
    public static byte[] getBytesByFile(String pathStr) {
        File file = new File(pathStr);
        try {
            FileInputStream fis = new FileInputStream(file);
            ByteArrayOutputStream bos = new ByteArrayOutputStream(1000);
            byte[] b = new byte[1000];
            int n;
            while ((n = fis.read(b)) != -1) {
                bos.write(b, 0, n);
            }
            fis.close();
            byte[] data = bos.toByteArray();
            bos.close();
            return data;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static String createFile(String interfaceName, String param,String message) throws IOException {
        File localFile = null;
        OutputStreamWriter write = null;
        BufferedWriter writer = null;
        try {
            //生成文件地址
            LocalDate date = LocalDate.now();
            String fileDirName = null;
            fileDirName = errorPath+"/" + date.getYear() + "/" + date.getMonthValue() + "/" + interfaceName;
            // 如果参数不为空时，再增加一级目录
            if(!ObjectUtils.isEmpty(param)){
                fileDirName +=  "_" + param;
            }
            //判断是否有该文件夹 否则创建
            File fileDir = new File(fileDirName);
            if (!fileDir.exists()) {
                fileDir.mkdirs();
            }
            String fileName = fileDir + "/" + UUID.randomUUID() + ".txt";
            localFile = new File(fileName);
            if (!localFile.exists()) {
                localFile.createNewFile();
            }
            write = new OutputStreamWriter(new FileOutputStream(localFile));
            writer = new BufferedWriter(write);
            writer.write(message);
            writer.flush();
        } catch (IOException e) {
            throw e;
        }finally {
            if(!ObjectUtils.isEmpty(write)){
                write.close();
            }
            if(!ObjectUtils.isEmpty(writer)){
                writer.close();
            }
        }
        return localFile.getAbsolutePath();
    }

    public static File createFile(String fileType, String inpNo, String visitId, String detailId, String tempPath) throws IOException {
        File localFile = null;
        try {
            //生成图片地址
            LocalDate date = LocalDate.now();
            String fileDirName = pdfPath + "/" + date.getYear() + "/" + date.getMonthValue() + "/" + inpNo + "_" + visitId;

            //判断是否有该文件夹 否则创建
            File fileDir = new File(fileDirName);
            if (!fileDir.exists()) {
                fileDir.mkdirs();
            }

            String fileName = null;
            if (Objects.equals(fileType, "images")) {
                fileName = fileDir + "/" + tempPath + detailId + ".jpg";
            } else if (Objects.equals(fileType, "pdfs")) {
                fileName = fileDir + "/"  + detailId + ".pdf";
            }

            localFile = new File(fileName);
            if (!localFile.exists()) {
                localFile.createNewFile();
            }

        } catch (IOException e) {
            throw e;
        }
        return localFile;
    }

    public static void main(String[] args) throws ClassNotFoundException {
        String address = "ftp://10.6.0.155/2019-10-21/000581031700_1_000581031700_0_1_JHR06.00.02_34_2_0.pdf";
        String a = address.substring(0, address.indexOf(".pdf"));
        String emrPath = a.substring(a.lastIndexOf("_") + 1, a.length());

    }


}
