package com.ann.demo.utils;

import com.alibaba.fastjson.JSON;
import com.ann.demo.entity.constant.AliasName;
import com.ann.demo.entity.constant.InterfaceName;
import com.ann.demo.entity.filing.dto.MessageDto;
import com.ann.demo.entity.interfaceEntity.*;
import org.apache.axis.utils.StringUtils;
import org.dom4j.*;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLFilterImpl;

import javax.annotation.PostConstruct;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.io.*;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Type;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description
 * @Date 2019/6/17 15:14
 * @Created by  ljx
 */
@Component
public class XMLUtils {

    static final Logger logger = LoggerFactory.getLogger(XMLUtils.class);

    private static final String DEFAULT_ENCODING = "UTF-8";

    private static final Map<String, String> map;

    static {
        map = new HashMap<String, String>();
        map.put("ns", "urn:hl7-org:v3");
    }

    public static String path;
    @Value("${file.path}")
    private String pathTemp;

    // 检验报告
    public static String inspectionReportPath;
    @Value("${file.inspectionReportPath}")
    private String inspectionReportPathTemp;

    // 检验申请单
    public static String inspectionApplyPath;
    @Value("${file.inspectionApplyPath}")
    private String inspectionApplyPathTemp;

    // 检验申请单确立
    public static String verificationInspectionApplyPath;
    @Value("${file.verificationInspectionApplyPath}")
    private String verificationInspectionApplyPathTemp;

    @PostConstruct
    public void init() {
        path = pathTemp;
        inspectionReportPath = inspectionReportPathTemp;
        inspectionApplyPath = inspectionApplyPathTemp;
        verificationInspectionApplyPath = verificationInspectionApplyPathTemp;
    }

    /**
     * 修改V3xml内容
     */
    public static String generateV3XmlStr(String typeCodeValue, String textContent) throws Exception{
        try {
            SAXReader reader = new SAXReader();
            Document document = reader.read(path);
            Element root = document.getRootElement();

            Element creationTime = root.element("creationTime");
            Attribute value = creationTime.attribute("value");
            value.setValue(DateUtils.getDate());

            Element acknowledgement = root.element("acknowledgement");
            Attribute typeCode = acknowledgement.attribute("typeCode");
            typeCode.setValue(typeCodeValue);

            Element acknowledgementDetail = acknowledgement.element("acknowledgementDetail");
            Element text = acknowledgementDetail.element("text");
            text.setText(textContent);
            return document.getRootElement().asXML();
        }catch (Exception e){
            return null;
        }
    }

    //--------------------------------------------向外抛出异常
    public static MessageDto xpathToBean(String xmlStr, Class tClass, String source) {
        MessageDto messageDto = null;
        Document doc = null;
        Object obj = null;
        try {
            xmlStr = xmlStr.replaceAll("&", "&amp;");
            SAXReader saxReader = new SAXReader();
            saxReader.setXMLFilter(new XMLFilterImpl() {
                @Override
                public void characters(char[] ch, int start, int length) throws SAXException {
                    String text = new String(ch, start, length);
                    if (length == 1) {
                        if ((int) ch[0] == 160) {
                            char[] escape = " ".toCharArray();
                            super.characters(escape, 0, escape.length);
                            return;
                        }
                    }
                    super.characters(ch, start, length);
                }
            });
            doc = saxReader.read(new ByteArrayInputStream(xmlStr.getBytes("UTF-8")), "UTF-8");
            if (tClass != null) {
                messageDto = getInstance(source);
                obj = tClass.newInstance();
            } else {
                // 是检查报告的接口
                String str = "/ns:ClinicalDocument/ns:component/ns:structuredBody/ns:component/ns:section/ns:entry/ns:observation/ns:code[@displayName='检查报告科室']/../ns:value/text()";
                //-----------------------存在一个错误 如果str找不到 就会向外抛出异常org.apache.cxf.interceptor.Fault: org/jaxen/JaxenException
                XPath x = doc.createXPath(str);
                x.setNamespaceURIs(map);

                Node node = x.selectSingleNode(doc);
                if (node != null) {
                    messageDto = getInstance(node.getText());
                    if (messageDto != null) {
                        obj = messageDto.getObject();
                    }
                }
            }
            if (obj != null && messageDto != null) {
                if (messageDto.getPropertiesPath() != null) {
                    // 加载配置文件
                    InputStream inputStream = new BufferedInputStream(new FileInputStream(new File(messageDto.getPropertiesPath())));
                    Properties properties = new Properties();
                    //加载格式化后的流
                    properties.load(new InputStreamReader(inputStream, DEFAULT_ENCODING));

                    Enumeration<?> enumeration = properties.propertyNames();
                    while (enumeration.hasMoreElements()) {
                        String key = (String) enumeration.nextElement();
                        String xpath = properties.getProperty(key);
                        XPath x = doc.createXPath(xpath);
                        x.setNamespaceURIs(map);
                        // 根据properties的value获取xml的value
                        String value = "";
                        Node node = x.selectSingleNode(doc);
                        if (node != null) {
                            value = node.getText();
                        }
                        // 判断属性是否存在
                        String objField = existsField(obj.getClass(), key);
                        if (objField != null) {
                            PropertyDescriptor descriptor = new PropertyDescriptor(key, obj.getClass());
                            if (("class java.util.Date").equals(objField)) {
                                if (value != null && value.length() > 0) {
                                    // 20200526 将时间格式定义成局部变量
                                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
                                    Date parse = simpleDateFormat.parse(value, new ParsePosition(0));
                                    descriptor.getWriteMethod().invoke(obj, parse);
                                }
                            } else {
                                descriptor.getWriteMethod().invoke(obj, value);
                            }
                        }
                        String messageField = existsField(messageDto.getClass(), key);
                        if (messageField != null) {
                            PropertyDescriptor propertyDescriptor = new PropertyDescriptor(key, messageDto.getClass());
                            // 给messageDto赋值
                            if (("class java.util.Date").equals(messageField)) {
                                if (value != null && value.length() > 0) {
                                    // 20200526 将时间格式定义成局部变量
                                    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
                                    propertyDescriptor.getWriteMethod().invoke(messageDto, simpleDateFormat.parse(value, new ParsePosition(0)));
                                }
                            } else {
                                propertyDescriptor.getWriteMethod().invoke(messageDto, value);
                            }
                        }
                    }
                    // 将对象转成json
                    messageDto.setObject(JSON.toJSONString(obj));
                    Map<String, String> map1 = new HashMap<>();
                    //重置pdf的base64为reportAddress节点的text
                    if(xmlStr.contains("广州金域") && StringUtils.isEmpty(messageDto.getReportAddress())){
                        parseXml2Map(xmlStr, map1);
                        String reportAddress = map1.get("component.structuredBody.component.section.pdf");
                        messageDto.setReportAddress(reportAddress);
                    }
                    return messageDto;
                }
            }
        } catch (DocumentException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (UnsupportedEncodingException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (FileNotFoundException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (IOException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (IllegalAccessException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (IntrospectionException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (InvocationTargetException e) {
            logger.error("出错咯！错误信息:" + e);
        } catch (InstantiationException e) {
            logger.error("出错咯！错误信息:" + e);
        }
        return null;
    }
    public static Map<String, String> parseXml2Map(String xml, Map<String, String> map) {
        try {
            SAXReader reader = new SAXReader();
            Document doc = reader.read(new StringReader(xml));
            Element root = doc.getRootElement();
            String path = "";
            if (map.containsKey(root.getName().trim())) {
                path = map.get(root.getName().trim());
                map.remove(root.getName().trim());
            }
            for (Iterator i = root.elementIterator(); i.hasNext();) {
                Element element = (Element) i.next();
                if (element.isTextOnly()) {
                    if (path.length() > 0) {
                        map.put(path + element.getName().trim(), element.getTextTrim());
                    } else {
                        map.put(element.getName().trim(), element.getTextTrim());
                    }
                } else {
                    map.put(element.getName().trim(), path+ element.getName().trim() + ".");
                    parseXml2Map(element.asXML(), map);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return map;
    }
    public static String replaceXpath(String xmlStr, String replaceContent) throws Exception{
        Document doc = null;
        try {
            xmlStr = xmlStr.replaceAll("&", "&amp;");
            SAXReader saxReader = new SAXReader();
            saxReader.setXMLFilter(new XMLFilterImpl() {
                @Override
                public void characters(char[] ch, int start, int length) throws SAXException {
                    String text = new String(ch, start, length);
                    if (length == 1) {
                        if ((int) ch[0] == 160) {
                            char[] escape = " ".toCharArray();
                            super.characters(escape, 0, escape.length);
                            return;
                        }
                    }
                    super.characters(ch, start, length);
                }
            });
            doc = saxReader.read(new ByteArrayInputStream(xmlStr.getBytes("UTF-8")), "UTF-8");

            XPath x = doc.createXPath("/ns:ClinicalDocument/ns:component/ns:structuredBody/ns:component[3]/ns:section/ns:pdf/@value");
            x.setNamespaceURIs(map);
            Node node = x.selectSingleNode(doc);
            if (node != null) {
                node.setText(replaceContent);
            }
        }catch (Exception e){
            return null;
        }
        return doc.asXML();
    }

    public static MessageDto getInstance(String source)  {
        MessageDto messageDto = null;
        // 根据source判断是哪个实体类
        switch (source) {
            case InterfaceName.INSPECTION_REPORT: // 检验报告
                messageDto = new MessageDto(UltrasonicReport.class.getName(), new UltrasonicReport(), AliasName.INSPECTION_REPORT, inspectionReportPath, source);
                break;
            case InterfaceName.PARAMETER_INSPECTIONAPPLY: // 检验报告申请单
                messageDto = new MessageDto(ExamApply.class.getName(), new ExamApply(), AliasName.PARAMETER_INSPECTIONAPPLY, inspectionApplyPath, source);
                break;
            case InterfaceName.PARAMETER_VERIFICATIONINSPECTIONAPPLY: // 检验报告申请单 核收检体
                messageDto = new MessageDto(ExamApply.class.getName(), new ExamApply(), AliasName.PARAMETER_VERIFICATIONINSPECTIONAPPLY, verificationInspectionApplyPath, source);
                break;

//            case InterfaceName.PARAMETER_VERIFICATIONINSPECTIONAPPLY: // 检验申请单 确立
//                messageDto = new MessageDto(ExamApply.class.getName(), new ExamApply(), AliasName.PARAMETER_INSPECTIONAPPLY, inspectionApplyPath, source);
//                break;
        }
        return messageDto;
    }

    public static String existsField(Class clz, String fieldName) {
        try {
            Field declaredField = clz.getDeclaredField(fieldName);
            if (declaredField != null) {
                return declaredField.getGenericType().toString();
            }
        } catch (Exception e) {
        }
        return null;
    }

}